@@ -79,7 +79,7 @@ class HelloWorldDataModule(LightningDataModule):
     A data module that gives the training, validation and test data for a simple 1-dim regression task.
     """
 
-    def __init__(self, crossval_count: int, crossval_index: int) -> None:
+    def __init__(self, crossval_count: int, crossval_index: Optional[int]) -> None:
         super().__init__()
         n_total = 200
         xy = _create_1d_regression_dataset(n=n_total)
@@ -92,6 +92,7 @@ class HelloWorldDataModule(LightningDataModule):
         else:
             # This could be done via a library function like sklearn's KFold function, but we don't want to add
             # scikit-learn as a dependency just for this example.
+            assert crossval_index is not None, "crossval_index must be set if crossval_count > 1"
             train, val = _split_crossval(xy[n_test:], crossval_count=crossval_count, crossval_index=crossval_index)
             self.val = HelloWorldDataset(xy=val)
             self.train = HelloWorldDataset(xy=train)
@@ -122,7 +123,7 @@ class HelloRegression(LightningModule):
         self.model = torch.nn.Linear(in_features=1, out_features=1, bias=True)  # type: ignore
         self.test_mse: List[torch.Tensor] = []
         self.test_mae = MeanAbsoluteError()
-        self.run_extra_val_epoch = False
+        self._on_extra_val_epoch = False
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:  # type: ignore
         """
@@ -232,6 +233,9 @@ class HelloRegression(LightningModule):
         Path("test_mae.txt").write_text(str(self.test_mae.compute().item()))
         self.log("test_mse", average_mse, on_epoch=True, on_step=False)
 
+    def on_run_extra_validation_epoch(self) -> None:
+        self._on_extra_val_epoch = True
+
 
 class HelloWorld(LightningContainer):
     """
@@ -272,3 +276,6 @@ class HelloWorld(LightningContainer):
                     *super().get_callbacks()]
         else:
             return super().get_callbacks()
+
+    def get_additional_aml_run_tags(self) -> Dict[str, str]:
+        return {"max_epochs": str(self.max_epochs)}
